# -*-coding: utf-8
"""
    flask_roles
    ~~~~~~~~~~~~~

    Adds Roles support to a flask project
"""


from .model import GroupMixin, RoleMixin, UserMixin

__all__ = ["Roles", "RoleMixin", "UserMixin", "GroupMixin"]


class Roles(object):
    """This class implements role-based access control module in Flask. There
    are two way to initialize Flask-Roles::

        app = Flask(__name__)
        roles = Roles(app)

    or::

        roles = Roles
        def create_app():
            app = Flask(__name__)
            roles.init_app(app)
            return app

    :param app: the Flask object
    """

    def __init__(self, app=None):
        """Initialize with app."""
        if app is not None:
            self.app = app
            self.init_app(app)
        else:
            self.app = None

    def init_app(self, app):
        """Initialize application in Flask-Roles. Adds (Role, app) to flask
        extensions.

        :param app: Flask object
        """



class RoleMixin(object):
    def get_children(self):
        yield self
        for child in self.children:
            yield child
            for grandchild in child.get_children():
                yield grandchild


class UserMixin(object):
    def add_role(self, role):
        self.roles.append(role)

    def add_roles(self, roles):
        for role in roles:
            self.add_role(role)

    def get_roles(self):
        # Traverse any role which has this role
        # as an ancestor
        for role in self.roles:
            yield role
            for child_role in role.get_children():
                yield child_role

    def get_role_names(self):
        for role in self.get_roles():
            yield role.name


class GroupMixin(UserMixin):
    pass



"""
Flask-Roles
-------------

A small plugin to flask to add roles support.

"""
from setuptools import setup

with open("README.rst", "r") as f:
    readme = f.read()

setup(
    name="flask_roles",
    version="0.5",
    url="https://github.com/labeneator/flask-roles",
    license="GPLv3",
    author="Laban Mwangi",
    author_email="support@east36.co.ke",
    description="Flask plugin to add roles to a project",
    long_description=readme,
    packages=["flask_roles"],
    zip_safe=False,
    include_package_data=True,
    extras_require={
        "testing": [
            "pip-tools == 4.5.1",
            "mccabe",
            "pytest-cov",
            "pytest",
            "Flask-Login>=0.5.0",
            "Flask-Principal>=0.4.0",
        ]
    },
    platforms="any",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
